#include <windows.h>
#include <stdio.h>

#define EXPORT __declspec(dllexport)

#include "../include/dwterm.h"



DWTRMF *dwt;



// y"=-sin(x), y(0)=0, y'(0)=0
// Analytic Solution: y'=cos(x)-1, y=sin(x)-x
//
// Let's solve the following equivalent equations,
//             numerically by predictor-corrector method.
// yp'=-sin(x), yp[0]=0
// y'=yp, y[0]=0


typedef struct
   {
   double y, yp;
   int n; // iterations
   }
D_ODE;

D_ODE *ode=NULL;

#define x1  0.0
#define x2  8.0
#define dy     0.1
#define dyp    0.1

int m=0,  // array size
    N=10; // maximum of n
double h=0.0,  // step=(x2-x1)/m
       r=0.1;  // Iterations stop when y, dy converge within r*dy and r*dyp.



//***********************************************************************
//                        Load & Unload
//***********************************************************************


EXPORT char *dwTermH_Load (char *CmdLine, char **title, DWTRMF *dwtf)
   {
   dwt=dwtf;

   dwt->ShowDisplay(FALSE);
          // Show the text window instead of the graphics window.

   return NULL;
   }


EXPORT char *dwTermH_Unload (void)
   {
   if(ode) free(ode);

   return NULL;
   }



//***********************************************************************
//                           Command
//***********************************************************************


#define RetParam { \
   dwt->Printf(1, "m = %d,  h = %8.5f,  r = %6.4f,  N = %d", m, h, r, N); \
   return NULL; \
}


#define RetErrorWrongNumber { \
   return "Error! Wrong number of tokens."; \
}


#define RetErrorWrongToken { \
   return "Error! A wrong token."; \
}


EXPORT char *dwTermH_Command (int nToken, char **token)
   {

   int i;
   if(nToken<=0) return NULL;

   if(!strcmp(token[0], "m"))
      { // Allocate data space and calculate h.
      if(nToken==2)
         {
         m=atoi(token[1]);

         if(ode) free(ode);
         ode=calloc(m, sizeof(D_ODE));
         h=(x2-x1)/m;

         RetParam
         }
      RetErrorWrongNumber
      }

   if(!strcmp(token[0], "r"))
      { // Specify r.
      if(nToken==2)
         {
         r=atof(token[1]);
         RetParam
         }
      RetErrorWrongNumber
      }

   if(!strcmp(token[0], "N"))
      { // Specify N.
      if(nToken==2)
         {
         N=atoi(token[1]);
         RetParam
         }
      RetErrorWrongNumber
      }

   if(!strcmp(token[0], "s"))
      { // Solve the equation.
      if(nToken==1)
         {
         double x,
                ypD, yD,
                ypC, yC;
         int n;
         for(i=1; i<m; i++)
            {
            x=x1+i*h;
            // predictor
            ypD=ode[i-1].yp-sin(x-h)*h;
            yD=ode[i-1].y+ode[i-1].yp*h;
            // corrector
            n=0;
            do
               {
               ypC=ode[i-1].yp-(sin(x-h)+sin(x))*h/2;
               yC=ode[i-1].y+(ode[i-1].yp+ypD)*h/2;
               n++;
               if( fabs((ypC-ypD)/dyp)<r &&
                   fabs((yC-yD)/dy)<r ) break;
               ypD=ypC; yD=yC;
               }
            while(n<=N);
            if(n>N)
               {
               dwt->Print(1, "Error! More iterations needed.");
               break;
               }
            else
               {
               ode[i].yp=ypC; ode[i].y=yC; ode[i].n=n;
               }
            }

         // Save the result into the file.
         int k;
         FILE *hFile=fopen("data", "wb");
         for(k=0; k<i; k++) dwt->Writef(hFile, "\r\n%8.5f  %8.5f %8.5f  %d",
                                                   x1+k*h, ode[k].y, ode[k].yp, ode[k].n);
         fclose(hFile);
         dwt->Print(1, "File saved.");
         return NULL;
         }
      RetErrorWrongNumber
      }
   RetErrorWrongToken                   
   }

