
//***********************************************************************
//                     Inputs and Outputs: v2.3
//***********************************************************************


typedef int DWTRMF_Write (void *hFile, char *str);
typedef int DWTRMF_Writef (void *hFile, char *format, ...);


typedef double *DWTRMF_ReadNumber (double *number, void *hFile);



//***********************************************************************
//                            Display
//***********************************************************************


typedef void DWTRMF_ShowDisplay (BOOL fGraphics); // v2.3
typedef BOOL DWTRMF_Print (int nLine, char *text); // v2.3
typedef BOOL DWTRMF_Printf (int nLine, char *format, ...); // v2.3


typedef void DWTRMF_SetDisplaySize (int width, int height);

typedef void DWTRMF_FitDisplay (void);

typedef void DWTRMF_CopyDisplay (void);


typedef HWND DWTRMF_GetDisplay (void);
typedef HWND DWTRMF_GetTextDisplay (void); // v2.3


typedef void DWTRMF_AnaglyphColors (int clrLeft, int clrRight);

typedef int DWTRMF_GetLeftColor (void);

typedef int DWTRMF_GetRightColor (void);



//***********************************************************************
//                             Canvases
//***********************************************************************


typedef struct {
   HDC hDC;
   HBITMAP hBm, hOldBm;
   POINT size;
} DWTRMD_Canvas;


typedef void DWTRMF_DeleteCanvas (DWTRMD_Canvas *cv);

typedef DWTRMD_Canvas *DWTRMF_CreateCanvas (int size_x, int size_y);


typedef void DWTRMF_RegisterCanvas (DWTRMD_Canvas *cv, BOOL fModel);

typedef RECT *DWTRMF_CopyCanvasRect (RECT *Rect, RECT *rc);

typedef DWTRMD_Canvas *DWTRMF_CloneCanvas (BOOL fCopy);


typedef void DWTRMF_AnaglyphCanvas (DWTRMD_Canvas *cv);

typedef DWTRMD_Canvas *DWTRMF_GetLeftCanvas (void);

typedef DWTRMD_Canvas *DWTRMF_GetRightCanvas (void);



//***********************************************************************
//                              Drawing
//***********************************************************************


typedef BOOL DWTRMF_NewPaintingBox (int num);

typedef int DWTRMF_GetPaintingBoxSize (void);

typedef BOOL DWTRMF_IsPaintingTool (int iTool);

typedef BOOL DWTRMF_FreePaintingTool (int iTool);

typedef BOOL DWTRMF_NewBrush (int iTool, HBRUSH hBr);

typedef BOOL DWTRMF_NewPen (int iTool, HPEN hPen);

typedef BOOL DWTRMF_NewFont (int iTool, HFONT hFont);

typedef BOOL DWTRMF_SelectBrush (int iTool);

typedef BOOL DWTRMF_SelectPen (int iTool);

typedef BOOL DWTRMF_SelectFont (int iTool);


typedef BOOL DWTRMF_SolidRect (RECT *rc, int iBr);

typedef BOOL DWTRMF_WireRect (RECT *rc, int iBr);

typedef BOOL DWTRMF_PaintDot (int x, int y, int dx, int dy, int iBr);


typedef void DWTRMF_Show (RECT *rc, int iBr);



//***********************************************************************
//                              Areas
//***********************************************************************


typedef struct {
   int sep;
   RECT rc;
} DWTRMD_Area;

typedef void DWTRMF_SetArea (DWTRMD_Area *area, int offset,
                             int left, int top, int right, int bottom);

typedef void DWTRMF_MoveArea (DWTRMD_Area *area, int doffset,
                                                       int dx, int dy);

typedef int DWTRMF_LandArea (DWTRMD_Area *area);

typedef void DWTRMF_ShowArea (DWTRMD_Area *area, int iBr);

typedef DWTRMD_Area *DWTRMF_CopyArea (DWTRMD_Area *areaDest, DWTRMD_Area *areaSrc);


typedef BOOL DWTRMF_NewAreaStack (int num);

typedef BOOL DWTRMF_PushArea (DWTRMD_Area *area);

typedef BOOL DWTRMF_PopArea (DWTRMD_Area *area);



//***********************************************************************
//                              Modeling
//***********************************************************************


typedef struct {
   DWTRMD_Area area;
   BOOL fOrtho;
   double width, height, zNear, zFar;
} DWTRMD_View;

typedef void DWTRMF_SetViewProjection (DWTRMD_View *view, BOOL fOrtho,
                    double width, double height, double zNear, double zFar);

typedef BOOL DWTRMF_BeginModel (DWTRMD_View *view,
                                          BOOL fStereo, double zScreen);

typedef void DWTRMF_EndModel (void);

typedef DWTRMD_View *DWTRMF_IsModeling (void);



//***********************************************************************
//                              Stop
//***********************************************************************


typedef void DWTRMF_EnableStop (char *str, char *title, char *text);

typedef void DWTRMF_DisableStop (void);

typedef BOOL DWTRMF_CanStop (void);

typedef char *DWTRMF_GetStopString (void);

typedef char *DWTRMF_GetStopTitle (void);

typedef char *DWTRMF_GetStopText (void);



//***********************************************************************
//               Function Pointers' Structure
//***********************************************************************


typedef struct {


   // Display

   DWTRMF_SetDisplaySize *SetDisplaySize;
   DWTRMF_FitDisplay *FitDisplay;
   DWTRMF_CopyDisplay *CopyDisplay;
   DWTRMF_GetDisplay *GetDisplay;

   DWTRMF_AnaglyphColors *AnaglyphColors;
   DWTRMF_GetLeftColor *GetLeftColor;
   DWTRMF_GetRightColor *GetRightColor;


   // Canvases

   DWTRMF_DeleteCanvas *DeleteCanvas;
   DWTRMF_CreateCanvas *CreateCanvas;

   DWTRMF_RegisterCanvas *RegisterCanvas;
   DWTRMF_CopyCanvasRect *CopyCanvasRect;
   DWTRMF_CloneCanvas *CloneCanvas;

   DWTRMF_AnaglyphCanvas *AnaglyphCanvas;
   DWTRMF_GetLeftCanvas *GetLeftCanvas;
   DWTRMF_GetRightCanvas *GetRightCanvas;


   // Drawing

   DWTRMF_NewPaintingBox *NewPaintingBox;
   DWTRMF_GetPaintingBoxSize *GetPaintingBoxSize;
   DWTRMF_IsPaintingTool *IsPaintingTool;
   DWTRMF_FreePaintingTool *FreePaintingTool;
   DWTRMF_NewBrush *NewBrush;
   DWTRMF_NewPen *NewPen;
   DWTRMF_NewFont *NewFont;
   DWTRMF_SelectBrush *SelectBrush;
   DWTRMF_SelectPen *SelectPen;
   DWTRMF_SelectFont *SelectFont;

   DWTRMF_SolidRect *SolidRect;
   DWTRMF_WireRect *WireRect;
   DWTRMF_PaintDot *PaintDot;

   DWTRMF_Show *Show;


   // Areas

   DWTRMF_SetArea *SetArea;
   DWTRMF_MoveArea *MoveArea;
   DWTRMF_LandArea *LandArea;
   DWTRMF_ShowArea *ShowArea;
   DWTRMF_CopyArea *CopyArea;

   DWTRMF_NewAreaStack *NewAreaStack;
   DWTRMF_PushArea *PushArea;
   DWTRMF_PopArea *PopArea;


   // Modeling

   DWTRMF_SetViewProjection *SetViewProjection;
   DWTRMF_BeginModel *BeginModel;
   DWTRMF_EndModel *EndModel;
   DWTRMF_IsModeling *IsModeling;


   // Stop

   DWTRMF_EnableStop *EnableStop;
   DWTRMF_DisableStop *DisableStop;
   DWTRMF_CanStop *CanStop;
   DWTRMF_GetStopString *GetStopString;
   DWTRMF_GetStopTitle *GetStopTitle;
   DWTRMF_GetStopText *GetStopText;


   //--- v2.3


   // Inputs and Outputs

   DWTRMF_Write *Write;
   DWTRMF_Writef *Writef;

   DWTRMF_ReadNumber *ReadNumber;


   // Display

   DWTRMF_ShowDisplay *ShowDisplay;
   DWTRMF_Print *Print;
   DWTRMF_Printf *Printf;

   DWTRMF_GetTextDisplay *GetTextDisplay;


} DWTRMF;




//***********************************************************************
//                             Handlers
//***********************************************************************


char *dwTermH_Load (char *CmdLine, char **title, DWTRMF *dwtf);

char *dwTermH_Unload (void);

char *dwTermH_Command (int nToken, char **token);


