#include <windows.h>

#define EXPORT __declspec(dllexport)

#include "../include/dwterm.h"

#include <GL/gl.h>
#include <GL/glu.h>


DWTRMF *dwt;

DWTRMD_Canvas *cv;



//***********************************************************************
//                       Modeling
//***********************************************************************


DWTRMD_View view;

GLUquadricObj *qobj;


double z,      // The distance from the viewer.
       length; // The sizes of the objects and relative distances 
               //    between them are based on this.


void model (void)
   {

   dwt->BeginModel(&view, FALSE, 0);

   // Erase
   glClearColor(0.0, 0.0, 0.0, 0.0);
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   // Lighting
   GLfloat white_light[]={1, 1, 1, 1};
   glLightfv(GL_LIGHT0, GL_DIFFUSE, white_light);
   GLfloat ambient[]={1, 1, 1, 1};
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
   GLfloat position[]={-1, 1, 1, 0};
   glLightfv(GL_LIGHT0, GL_POSITION, position);
   glEnable(GL_LIGHT0);
   glEnable(GL_LIGHTING);
   glEnable(GL_DEPTH_TEST);

   // The common responses to the light, of the two objects
   glShadeModel(GL_FLAT);
   GLfloat mat_specular[]={1.0, 1.0, 1.0, 1.0};
   GLfloat mat_shininess[]={30.0};
   glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
   glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);

   // The objects are quadric and their surfaces are solid.
   gluQuadricDrawStyle(qobj, GLU_FILL);

   // Where to build the objects
   glPushMatrix();
   glTranslatef(0, 0, -z);

   // Build the sphere.
   glPushMatrix();
      glTranslatef(-length*0.7, 0, 0);
      glRotatef(30, -1, 0, 0);
      GLfloat blue[]={0, 0, 1, 1};
      glMaterialfv(GL_FRONT, GL_DIFFUSE, blue);
      gluSphere(qobj, length, 100, 100);
   glPopMatrix();

   // Build the cylinder.
   glPushMatrix();
      glTranslatef(0, -length*0.5, length*2.5);
      glRotatef(45, -1, 1, 0);
      GLfloat green[]={0, 1, 0, 1};
      glMaterialfv(GL_FRONT, GL_DIFFUSE, green);
      gluCylinder(qobj, length*0.5, length*0.5, length*1.5, 50, 50);
   glPopMatrix();

   glPopMatrix();

   dwt->EndModel();

   }



//***********************************************************************
//                        Load & Unload
//***********************************************************************



EXPORT char *dwTermH_Load (char *CmdLine, char **Title, DWTRMF *dwtf)
   {

   dwt=dwtf;

   // Create and register the canvases.
   cv=dwt->CreateCanvas(400, 300);
   dwt->RegisterCanvas(cv, TRUE);
   dwt->FitDisplay();

   // Initialize the view.
   dwt->CopyCanvasRect(&(view.area.rc), NULL);
   dwt->SetViewProjection(&view, FALSE, 400, 300, 500, 10000);

   // initialize the modeling.
   qobj=gluNewQuadric();
   length=1200;
   z=7000;
   model();

   return NULL;

   }


EXPORT char *dwTermH_Unload (void)
   {
   gluDeleteQuadric(qobj);
   dwt->DeleteCanvas(cv);
   return NULL;
   }



//***********************************************************************
//                           Command
//***********************************************************************


#define RetParam { \
   sprintf(Result, "z = %f", z); \
   return Result; \
}


#define RetErrorWrongNumber { \
   return "Error! Wrong number of tokens."; \
}


#define RetErrorWrongToken { \
   return "Error! A wrong token."; \
}


char Result[256]; // Buffer for displaying a message on the status bar.


EXPORT char *dwTermH_Command (int nToken, char **token)
   {

   int i;
   if(nToken<=0) return NULL;

   if(!strcmp(token[0], "z"))
      {           
      if(nToken==1) RetParam //Just show z on the status bar.
      if(nToken==2)
         {
         z=atof(token[1]);
         model();
         RetParam;
         }
      RetErrorWrongNumber
      }

   RetErrorWrongToken

   }

