#include <windows.h>

#define EXPORT __declspec(dllexport)

#include "../include/dwterm.h"



DWTRMF *dwt;

DWTRMD_Canvas *cv=NULL;

// Brush.

double Zero=0.5; 
// will be compared with Function to determine one out of the 3 bruses for the dot.

#define IPT_BkBr    0 //for erasing.
#define IPT_NegBr   1 //The 3 brush colors for dots.
#define IPT_ZeroBr  2
#define IPT_PosBr   3
#define IPT_Num        IPT_PosBr+1

int clr[4]={0xff0000,  0xaa0000, 0x00aa00, 0x0000aa};

// Plot.

double Axx=0.5, Axy=0, Ayy=1, Ax=0, Ay=0, A1=-5000;

double Function (POINT *pt)
   {
   return Axx*(pt->x)*(pt->x) +
          Axy*(pt->x)*(pt->y) +
          Ayy*(pt->y)*(pt->y) +
          Ax*(pt->x) + Ay*(pt->y) + A1;
   }

// Region.

int t=1; // width of the mathematical square dot, in GDI pixels.

POINT ptMin={-200, -150}, ptMax={199, 149};
// mathematical region. (coordinate system differs from GDI)



//***********************************************************************
//                    Routines for the canvas and painting
//***********************************************************************
/*
Note that the two routines Dot and Init use a "mathematical coordinate system",
where the vertical coordinate increases from bottom to top in contrast to GDI,
the origin can be changed by a user input. Therefore the routines take into
account not only the dot(mathematical point)-size and mathematical coordinate
ranges but also such a convention, before drawing.
*/


// Erase all on the canvas.
// Create the canvas if new mathematical region results new size.

void Init (void)
   {

   int w, h;
   w=t*(ptMax.x-ptMin.x+1); // Calculate new size.
   h=t*(ptMax.y-ptMin.y+1);

   if(cv)
      {
      if((cv->size.x==w)&&(cv->size.y==h))
         {
         dwt->Show(NULL, IPT_BkBr); // Just erase if nothing changed.
         return;
         }
      }

   dwt->Show(NULL, -1);   // Needed if new canvas cannot encompass the present one.
   //dwt->RegisterCanvas(NULL, FALSE); Not needed if any canvas is to exist 
                                                      // before the next update.
   dwt->DeleteCanvas(cv);

   cv=dwt->CreateCanvas(w, h);
   dwt->RegisterCanvas(cv, FALSE);
   dwt->FitDisplay();
   dwt->Show(NULL, IPT_BkBr);

   }


// Draw a dot.
// Note that PaintDot does not paint if the brush does not exist.
void Dot (int x, int y, int iBr)
   {
   dwt->PaintDot(t*(x-ptMin.x), t*(ptMax.y-y), t, t, iBr);
   }


//Paint all the dots.

void Draw (void)
   {
   POINT pt; double f;
   for(pt.x=ptMin.x; pt.x<=ptMax.x; pt.x++)
      {
      for(pt.y=ptMin.y; pt.y<=ptMax.y; pt.y++)
         {
         f=Function(&pt);
         if(f<(-Zero)) Dot(pt.x, pt.y, IPT_NegBr);
         if(fabs(f)<=Zero) Dot(pt.x, pt.y, IPT_ZeroBr);
         if(f>Zero) Dot(pt.x, pt.y, IPT_PosBr);
         }
      }
   }


/*

If you don't know why Show is called before DeleteCanvas in
the routine Init as well as after the creation, rebuild this example
after deleting the Show, to see what will happen:
   You will see the contents of the old large canvas remain as the
background of those of the new canvas unless the user (you) manually
updates the window (by minimizing and restoring the main window for
example), when you change t from 2 to 1 for example.
   The answer is that the parts of the old area in the window
which do not belong to the new area should be updated too so that
the window paints over them with the internal gray brush.

*/



//***********************************************************************
//                        Load & Unload
//***********************************************************************


EXPORT char *dwTermH_Load (char *CmdLine, char **Title, DWTRMF *dwtf)
   {

   dwt=dwtf;
   *Title="Plot 2D 2nd Order";

   dwt->NewPaintingBox(IPT_Num);
   dwt->NewBrush(IPT_BkBr, CreateSolidBrush(clr[IPT_BkBr]));

   Init();

   return NULL;
}


EXPORT char *dwTermH_Unload (void)
   {
   dwt->DeleteCanvas(cv);
   return NULL;
   }



//***********************************************************************
//                           Command
//***********************************************************************


// Error-returns.

#define RetErrorWrongNumber { \
   strcpy(Result, "Error! Wrong number of tokens."); \
   return Result; \
}

#define RetErrorWrongToken { \
   strcpy(Result, "Error! A wrong token."); \
   return Result; \
}

#define RetErrorWrongRegion { \
   strcpy(Result, "Error! Wrong region."); \
   return Result; \
}


// Success-returns, displaying relavant pieces of information.

void DisplayBrush (char *Result, char *Name, int  iBr)
   { // Information about a brush.

   char Buffer[MAX_PATH];
   strcat(Result, ", ");
   strcat(Result, Name);
   strcat(Result, ": ");

   if(!dwt->IsPaintingTool(iBr)) strcat(Result, "N(");
   sprintf(Buffer, "%06x", clr[iBr]);
   strcat(Result, Buffer);
   if(!dwt->IsPaintingTool(iBr)) strcat(Result, ")");

   }

#define RetBrush { \
   sprintf(Result, "z=%f", Zero); \
   DisplayBrush(Result, "-", IPT_NegBr); \
   DisplayBrush(Result, "0", IPT_ZeroBr); \
   DisplayBrush(Result, "+", IPT_PosBr); \
   DisplayBrush(Result, "e", IPT_BkBr); \
   return Result; \
}

#define RetParam { \
   sprintf(Result, "Axx=%.2f, Axy=%.2f, Ayy=%.2f, Ax=%.2f, Ay=%.2f, A1=%.2f",\
                    Axx, Axy, Ayy, Ax, Ay, A1); \
   return Result; \
}

#define RetRegion { \
   sprintf(Result, "t=%d, r=(%d %d %d %d)", t, \
                          ptMin.x, ptMin.y, ptMax.x, ptMax.y); \
   return Result; \
}


// macros for changing somethings, not only displaying and returning.

#define RetSpecBrush(iBr) { \
         if(nToken==2) { \
            dwt->NewBrush(iBr, CreateSolidBrush(clr[iBr])); \
            RetBrush} \
         if(0==strcmp(Token[2], "n")) { \
            if(nToken!=3) RetErrorWrongNumber \
            dwt->FreePaintingTool(iBr); \
            RetBrush} \
         if(0==strcmp(Token[2], "c")) { \
            if(nToken!=4) RetErrorWrongNumber \
            clr[iBr]=strtol(Token[3], NULL, 0); \
            dwt->NewBrush(iBr, CreateSolidBrush(clr[iBr])); \
            RetBrush} \
         RetErrorWrongToken \
}

#define RetSpecParam(Param) { \
            Param=atof(Token[2]); \
            RetParam \
}


char Result[MAX_PATH];

EXPORT char *dwTermH_Command (int nToken, char **Token) {
   POINT pt1, pt2;
   int i;
   if(nToken<=0) return NULL;
   // Brush.
   if(0==strcmp(Token[0], "z")) { // Define Zero-range and display about brushes.
      if(nToken==1) RetBrush // Just display.
      if(nToken==2) {
         Zero=atof(Token[1]);
         RetBrush}
      RetErrorWrongNumber}
   if(0==strcmp(Token[0], "b")) { // Specify, create, delete and display about brushes.
      if(nToken==1) RetBrush // Just display.
      if(0==strcmp(Token[1], "-")) RetSpecBrush(IPT_NegBr)
      if(0==strcmp(Token[1], "0")) RetSpecBrush(IPT_ZeroBr)
      if(0==strcmp(Token[1], "+")) RetSpecBrush(IPT_PosBr)
      if(0==strcmp(Token[1], "e")) RetSpecBrush(IPT_BkBr)
      RetErrorWrongToken}
   if(0==strcmp(Token[0], "e")) {// Erase and display about brushes.
      if(nToken==1) {
         Init();
         RetBrush}
      RetErrorWrongNumber}
   //Plot.
   if(0==strcmp(Token[0], "p")) { // Specify and display parameters.
      if(nToken==1) RetParam // Just display.
      if(nToken==3) {
         if(0==strcmp(Token[1], "xx")) RetSpecParam(Axx)
         if(0==strcmp(Token[1], "xy")) RetSpecParam(Axy)
         if(0==strcmp(Token[1], "yy")) RetSpecParam(Ayy)
         if(0==strcmp(Token[1], "x")) RetSpecParam(Ax)
         if(0==strcmp(Token[1], "y")) RetSpecParam(Ay)
         if(0==strcmp(Token[1], "1")) RetSpecParam(A1)
         RetErrorWrongToken}
      RetErrorWrongNumber}
   if(0==strcmp(Token[0], "d")) { // Draw the dots and display parameters.
      if(nToken==1) {
         Draw();
         dwt->Show(NULL, -1);
         RetParam}
      RetErrorWrongNumber}
   //Region.
   if(0==strcmp(Token[0], "t")) { // Define the dot-size, draw the dots and display about the region.
      if(nToken==1) RetRegion // Just display.
      if(nToken==2) {
         i=atoi(Token[1]);
         if(i!=t) {
            t=i;
            Init();
            Draw();}
         RetRegion}
      RetErrorWrongNumber}
   if(0==strcmp(Token[0], "r")) { // Define the region, draw the dots and display about the region.
      if(nToken==1) RetRegion // Just display.
      if(nToken==5) {
         pt1.x=atoi(Token[1]);
         pt1.y=atoi(Token[2]);
         pt2.x=atoi(Token[3]);
         pt2.y=atoi(Token[4]);
         if((pt1.x>pt2.x)||(pt1.y>pt2.y)) RetErrorWrongRegion // Wrong if any one pair is reversed.
         if( (pt2.x!=ptMax.x) || (pt1.x!=ptMin.x) ||
             (pt2.y!=ptMax.y) || (pt1.y!=ptMin.y) ) { //Draw if any one coordinate limit is changed.
            ptMin=pt1;
            ptMax=pt2;
            Init();
            Draw();}
         RetRegion}
      RetErrorWrongNumber}
   RetErrorWrongToken
}

