#include <windows.h>

#define EXPORT __declspec(dllexport)

#include "../include/dwterm.h"

#include <GL/gl.h>
#include <GL/glu.h>


DWTRMF *dwt;

DWTRMD_Canvas *cvl, *cvr;



//***********************************************************************
//                       Modeling
//***********************************************************************


DWTRMD_View view;

GLUquadricObj *qobj;


double zScreen, z, length;



void model (void)
   {

   dwt->BeginModel(&view, TRUE, zScreen);

   GLfloat ambient[]={1, 1, 1, 1};
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
   GLfloat position[]={0, 0, 1, 1};
   glLightfv(GL_LIGHT0, GL_POSITION, position);
   glEnable(GL_LIGHT0);
   glEnable(GL_LIGHTING);
   glEnable(GL_DEPTH_TEST);
   glShadeModel(GL_FLAT);

   glClearColor(0.0, 0.0, 0.0, 0.0);
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   glPushMatrix();
   glTranslatef(0, 0, -z);

   // Draw the sphere.

   glPushMatrix();
      glTranslatef(-length*0.9, length*0.3, 0);
      glRotatef(30, -1, 0, 0);
      gluQuadricDrawStyle(qobj, GLU_FILL);
      GLfloat colorSphere[]={1, 1, 0, 1};
      glMaterialfv(GL_FRONT, GL_DIFFUSE, colorSphere);
      gluSphere(qobj, length*1.2, 10, 6);
   glPopMatrix();

   // Draw the cylinder.

   glPushMatrix();
      glTranslatef(0, 0, length*2.5);
      glRotatef(40, 1, 1, 1);
      gluQuadricDrawStyle(qobj, GLU_LINE);
      glLineWidth(8);
      GLfloat colorCyl[]={1, 0, 1, 1};
      glMaterialfv(GL_FRONT, GL_DIFFUSE, colorCyl);
      gluCylinder(qobj, length*0.8, length*0.8, length*1.5, 15, 8);
   glPopMatrix();

   glPopMatrix();

   dwt->EndModel();

   }



//***********************************************************************
//                        Load & Unload
//***********************************************************************



EXPORT char *dwTermH_Load (char *CmdLine, char **Title, DWTRMF *dwtf)
   {

   dwt=dwtf;

   // Create and register the canvases.
   cvl=dwt->CreateCanvas(400, 300);
   dwt->RegisterCanvas(cvl, TRUE);
   dwt->FitDisplay();
   cvr=dwt->CloneCanvas(FALSE);
   dwt->AnaglyphCanvas(cvr);

   // Initialize the view.
   dwt->CopyArea(&(view.area), NULL);
   InflateRect(&(view.area.rc), 3, 0);
   dwt->MoveArea(&(view.area), 3, 0, 0);
   dwt->SetViewProjection(&view, FALSE, 400+3*2, 300, 500, 10000);
   qobj=gluNewQuadric();

   // initialize the modeling.
   zScreen=2500;
   length=1200;
   z=6500;
   model();

   return NULL;

   }


EXPORT char *dwTermH_Unload (void)
   {
   gluDeleteQuadric(qobj);
   dwt->DeleteCanvas(cvl);
   dwt->DeleteCanvas(cvr);
   return NULL;
   }



//***********************************************************************
//                           Command
//***********************************************************************


#define RetParam { \
   sprintf(Result, "z = %f,  s = %f", z, zScreen); \
   return Result; \
}


#define RetErrorWrongNumber { \
   return "Error! Wrong number of tokens."; \
}


#define RetErrorWrongToken { \
   return "Error! A wrong token."; \
}


char Result[256]; // Buffer for displaying a message on the status bar.




DWTRMD_Canvas cvtt;

EXPORT char *dwTermH_Command (int nToken, char **token)
   {

   int i;
   if(nToken<=0) return NULL;

   if(!strcmp(token[0], "z"))
      {
      if(nToken==1) RetParam //Just show z and zScreen on the status bar.
      if(nToken==2)
         {
         z=atof(token[1]);
         model();
         RetParam;
         }
      RetErrorWrongNumber
      }                   

   if(!strcmp(token[0], "s"))
      {
      if(nToken==1) RetParam //Just show z and zScreen on the status bar.
      if(nToken==2)
         {
         zScreen=atof(token[1]);
         model();
         RetParam;
         }
      RetErrorWrongNumber
      }

   if(!strcmp(token[0], "m"))
      {
      if(nToken==1)
         {
/*
         dwt->AnaglyphCanvas(NULL);
         model();
*/
cvtt.hDC=cvl->hDC;
cvtt.hOldBm=cvl->hOldBm;
cvtt.size.x=cvl->size.x;
cvtt.size.y=cvl->size.y;

cvl->hDC=cvr->hDC;
cvl->hOldBm=cvr->hOldBm;
cvl->size.x=cvr->size.x;
cvl->size.y=cvr->size.y;

dwt->Show(NULL, -1);


         return NULL;
         }
      RetErrorWrongNumber
      }       

   if(!strcmp(token[0], "a"))
      {
      if(nToken==1)
         {
         dwt->AnaglyphCanvas(cvr);
         model();
         return NULL;
         }
      RetErrorWrongNumber
      }       

   RetErrorWrongToken

   }

